@@ -25,6 +25,8 @@
 #include <string.h>
 #include <time.h>
 
+#include <glib.h>
+
 #include <camel/camel-folder-search.h>
 #include <camel/camel-mime-part.h>
 #include <camel/camel-mime-utils.h>
@@ -42,6 +44,7 @@
 #include <exchange-mapi-defs.h>
 #include <exchange-mapi-utils.h>
 #include <exchange-mapi-folder.h>
+#include <exchange-mapi-cal-utils.h>
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
@@ -71,7 +74,7 @@ typedef struct {
 /*For collecting summary info from server*/
 typedef struct {
 	GSList *items_list;
-	const struct timeval *last_modification_time;
+	GTimeVal last_modification_time;
 }fetch_items_data;
 
 static CamelMimeMessage *mapi_folder_item_to_msg( CamelFolder *folder, MapiItem *item, CamelException *ex );
@@ -186,7 +189,8 @@ fetch_items_cb (FetchItemsCallbackData *item_data, gpointer data)
 	long *flags;
 	struct FILETIME *delivery_date = NULL;
 	struct FILETIME *last_modification_time = NULL;
-	struct timeval *item_modification_time = NULL;
+	struct timeval item_modification_time = { 0 };
+	struct timeval fi_data_mod_time = { 0 };
 	guint32 j = 0;
 	NTTIME ntdate;
 
@@ -202,7 +206,7 @@ fetch_items_cb (FetchItemsCallbackData *item_data, gpointer data)
 
 	for (j = 0; j < item_data->properties->cValues; j++) {
 
-		gpointer prop_data = get_mapi_SPropValue_data(&item_data->properties->lpProps[j]);
+		gconstpointer prop_data = get_mapi_SPropValue_data(&item_data->properties->lpProps[j]);
 
 		switch (item_data->properties->lpProps[j].ulPropTag) {
 		/* FIXME : Instead of duping. Use talloc_steal to reuse the memory */
@@ -254,12 +258,16 @@ fetch_items_cb (FetchItemsCallbackData *item_data, gpointer data)
 		ntdate = last_modification_time->dwHighDateTime;
 		ntdate = ntdate << 32;
 		ntdate |= last_modification_time->dwLowDateTime;
-		item_modification_time = g_new0 (struct timeval, 1);
-		nttime_to_timeval(item_modification_time, ntdate);
+		nttime_to_timeval (&item_modification_time, ntdate);
 	}
 
-	if (timeval_compare (item_modification_time, fi_data->last_modification_time) == 1) 
-			fi_data->last_modification_time = item_modification_time;
+	fi_data_mod_time.tv_sec = fi_data->last_modification_time.tv_sec;
+	fi_data_mod_time.tv_usec = fi_data->last_modification_time.tv_usec;
+
+	if (timeval_compare (&item_modification_time, &fi_data_mod_time) == 1) {
+			fi_data->last_modification_time.tv_sec = item_modification_time.tv_sec;
+			fi_data->last_modification_time.tv_usec = item_modification_time.tv_usec;
+	}
 
 	if ((*flags & MSGFLAG_READ) != 0)
 		item->header.flags |= CAMEL_MESSAGE_SEEN;
@@ -306,6 +314,7 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelException *ex, gboole
 	for ( ; item_list != NULL ; item_list = g_slist_next (item_list) ) {
 		MapiItem *temp_item ;
 		MapiItem *item;
+		gchar *msg_uid;
 		guint64 id;
 
 		exists = FALSE;
@@ -322,7 +331,7 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelException *ex, gboole
 		/************************ First populate summary *************************/
 		mi = NULL;
 		pmi = NULL;
-		char *msg_uid = exchange_mapi_util_mapi_ids_to_uid (item->fid, item->mid);
+		msg_uid = exchange_mapi_util_mapi_ids_to_uid (item->fid, item->mid);
 		pmi = camel_folder_summary_uid (folder->summary, msg_uid);
 
 		if (pmi) {
@@ -335,7 +344,7 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelException *ex, gboole
 			mi = (CamelMapiMessageInfo *)camel_message_info_new (folder->summary); 
 			if (mi->info.content == NULL) {
 				mi->info.content = camel_folder_summary_content_info_new (folder->summary);
-				mi->info.content->type = camel_content_type_new ("multipart", "mixed");	
+				mi->info.content->type = camel_content_type_new ("multipart", "related");      
 			}
 		}
 		
@@ -437,9 +446,10 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			const char *uid;
 			mapi_id_t *mid = g_new0 (mapi_id_t, 1); /* FIXME : */
 			mapi_id_t temp_fid;
+			guint32 flags;
 
 			uid = camel_message_info_uid (info);
-			guint32 flags= camel_message_info_flags (info);
+			flags= camel_message_info_flags (info);
 
 			/* Why are we getting so much noise here :-/ */
 			if (!exchange_mapi_util_mapi_ids_from_uid (uid, &temp_fid, mid))
@@ -544,7 +554,7 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 	CamelMapiFolder *mapi_folder = CAMEL_MAPI_FOLDER (folder);
 	CamelMapiSummary *mapi_summary = CAMEL_MAPI_SUMMARY (folder->summary);
 	gboolean is_proxy = folder->parent_store->flags & CAMEL_STORE_PROXY;
-	gboolean is_locked = TRUE;
+	gboolean is_locked = FALSE;
 	gboolean status;
 
 	struct mapi_SRestriction *res = NULL;
@@ -565,8 +575,8 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 		PR_DISPLAY_BCC
 	};
 
-	if (((CamelOfflineStore *) mapi_store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) 
-		return;
+	if (((CamelOfflineStore *) mapi_store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
+		goto end1;
 
 	/* Sync-up the (un)read changes before getting updates,
 	so that the getFolderList will reflect the most recent changes too */
@@ -576,7 +586,7 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 	folder_id = camel_mapi_store_folder_id_lookup (mapi_store, folder->full_name);
 	if (!folder_id) {
 		d(printf ("\nERROR - Folder id not present. Cannot refresh info for %s\n", folder->full_name));
-		return;
+		goto end1;
 	}
 
 	if (camel_folder_is_frozen (folder) ) {
@@ -584,6 +594,7 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 	}
 
 	CAMEL_SERVICE_REC_LOCK (mapi_store, connect_lock);
+	is_locked = TRUE;
 
 	if (!camel_mapi_store_connected (mapi_store, ex))
 		goto end1;
@@ -593,10 +604,8 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 		mapi_id_t temp_folder_id;
 		guint32 options = 0;
 
-		fetch_data->last_modification_time = g_new0 (struct timeval, 1); /*First Sync*/
-
 		if (mapi_summary->sync_time_stamp && *mapi_summary->sync_time_stamp &&
-		    g_time_val_from_iso8601 (mapi_summary->sync_time_stamp, fetch_data->last_modification_time)) {
+		    g_time_val_from_iso8601 (mapi_summary->sync_time_stamp, &fetch_data->last_modification_time)) {
 			struct SPropValue sprop;
 			struct timeval t;
 
@@ -606,8 +615,8 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 			res->res.resProperty.relop = RELOP_GE;
 			res->res.resProperty.ulPropTag = PR_LAST_MODIFICATION_TIME;
 
-			t.tv_sec = fetch_data->last_modification_time->tv_sec;
-			t.tv_usec = fetch_data->last_modification_time->tv_usec;
+			t.tv_sec = fetch_data->last_modification_time.tv_sec;
+			t.tv_usec = fetch_data->last_modification_time.tv_usec;
 
 			//Creation time ? 
 			set_SPropValue_proptag_date_timeval (&sprop, PR_LAST_MODIFICATION_TIME, &t);
@@ -649,7 +658,7 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 		}
 
 		/*Preserve last_modification_time from this fetch for later use with restrictions.*/
-		mapi_summary->sync_time_stamp = g_time_val_to_iso8601 (fetch_data->last_modification_time);
+		mapi_summary->sync_time_stamp = g_time_val_to_iso8601 (&fetch_data->last_modification_time);
 
 		camel_folder_summary_touch (folder->summary);
 		mapi_sync_summary (folder, ex);
@@ -662,15 +671,15 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 	CAMEL_SERVICE_REC_UNLOCK (mapi_store, connect_lock);
 	is_locked = FALSE;
 
-	g_slist_foreach (fetch_data->items_list, (GFunc) mapi_item_free, NULL);
-	g_slist_free (fetch_data->items_list);
 end2:
 	//TODO:
 end1:
 	if (is_locked)
 		CAMEL_SERVICE_REC_UNLOCK (mapi_store, connect_lock);
-	return;
 
+	g_slist_foreach (fetch_data->items_list, (GFunc) mapi_item_free, NULL);
+	g_slist_free (fetch_data->items_list);
+	g_free (fetch_data);
 }
 
 static const uint32_t camel_GetPropsList[] = {
@@ -839,7 +848,7 @@ fetch_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 
 	for (j = 0; j < item_data->properties->cValues; j++) {
 
-		gpointer prop_data = get_mapi_SPropValue_data(&item_data->properties->lpProps[j]);
+		gconstpointer prop_data = get_mapi_SPropValue_data(&item_data->properties->lpProps[j]);
 
 		switch (item_data->properties->lpProps[j].ulPropTag) {
 		/*FIXME : Instead of duping. Use talloc_steal to reuse the memory*/
@@ -976,6 +985,8 @@ mapi_populate_details_from_item (CamelMimeMessage *msg, MapiItem *item)
 	char *temp_str = NULL;
 	time_t recieved_time;
 	CamelInternetAddress *addr = NULL;
+	int offset = 0;
+	time_t actual_time;
 
 	temp_str = item->header.subject;
 	if(temp_str) 
@@ -983,8 +994,7 @@ mapi_populate_details_from_item (CamelMimeMessage *msg, MapiItem *item)
 
 	recieved_time = item->header.recieved_time;
 
-	int offset = 0;
-	time_t actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
+	actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
 	camel_mime_message_set_date (msg, actual_time, offset);
 
 	if (item->header.from) {
@@ -1005,10 +1015,10 @@ static void
 mapi_populate_msg_body_from_item (CamelMultipart *multipart, MapiItem *item, ExchangeMAPIStream *body)
 {
 	CamelMimePart *part;
+	const char* type = NULL;
 
 	part = camel_mime_part_new ();
 	camel_mime_part_set_encoding(part, CAMEL_TRANSFER_ENCODING_8BIT);
-	const char* type = NULL;
 	
 	if (body) { 
 		if (item->is_cal)
@@ -1023,7 +1033,6 @@ mapi_populate_msg_body_from_item (CamelMultipart *multipart, MapiItem *item, Exc
 	} else
 		camel_mime_part_set_content(part, " ", strlen(" "), "text/html");
 
-	camel_multipart_set_boundary (multipart, NULL);
 	camel_multipart_add_part (multipart, part);
 	camel_object_unref (part);
 }
@@ -1036,6 +1045,7 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 {
 	CamelMimeMessage *msg = NULL;
 	CamelMultipart *multipart = NULL;
+	CamelContentType *content_type = NULL;
 
 	GSList *attach_list = NULL;
 	int errno;
@@ -1050,6 +1060,14 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 
 	multipart = camel_multipart_new ();
 
+	/*FIXME : Using set of default. Fix it during mimewriter*/
+	camel_data_wrapper_set_mime_type (CAMEL_DATA_WRAPPER (multipart),
+					  "multipart/related");
+	camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, 
+				     "type", "multipart/alternative");
+
+	camel_multipart_set_boundary (multipart, NULL);
+
 	camel_mime_message_set_message_id (msg, uid);
 	body_part_list = item->msg.body_parts;
 	while (body_part_list){
@@ -1068,29 +1086,41 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 		for (al = attach_list; al != NULL; al = al->next) {
 			ExchangeMAPIAttachment *attach = (ExchangeMAPIAttachment *)al->data;
 			ExchangeMAPIStream *stream = NULL;
-			const char *filename, *mime_type; 
+			const char *filename, *mime_type, *content_id = NULL; 
 			CamelMimePart *part;
 
-			filename = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, PR_ATTACH_LONG_FILENAME);
-			if (!(filename && *filename))
-				filename = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, PR_ATTACH_FILENAME);
-
-			mime_type = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, PR_ATTACH_MIME_TAG);
-
 			stream = exchange_mapi_util_find_stream (attach->streams, PR_ATTACH_DATA_BIN);
 
 			if (!stream || stream->value->len <= 0) {
 				continue;
 			}
+
 			part = camel_mime_part_new ();
 
+			filename = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
+												 PR_ATTACH_LONG_FILENAME);
+
+			if (!(filename && *filename))
+				filename = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
+													 PR_ATTACH_FILENAME);
 			camel_mime_part_set_filename(part, g_strdup(filename));
-			//Auto generate content-id
-			camel_mime_part_set_content_id (part, NULL);
-			camel_mime_part_set_content(part, stream->value->data, stream->value->len, mime_type);
 			camel_content_type_set_param (((CamelDataWrapper *) part)->mime_type, "name", filename);
 
-			camel_multipart_set_boundary(multipart, NULL);
+			/*Content-Type*/
+			mime_type = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
+												  PR_ATTACH_MIME_TAG);
+
+			camel_mime_part_set_content(part, stream->value->data, stream->value->len, mime_type);
+
+			/*Content-ID*/
+			content_id = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
+												   PR_ATTACH_CONTENT_ID);
+
+			camel_mime_part_set_content_id (part, content_id);
+
+			/*Fall back to default*/
+			camel_mime_part_set_encoding (part, CAMEL_TRANSFER_ENCODING_BASE64);
+
 			camel_multipart_add_part (multipart, part);
 			camel_object_unref (part);
 			
@@ -1119,6 +1149,11 @@ mapi_folder_get_message( CamelFolder *folder, const char *uid, CamelException *e
 	CamelStream *stream, *cache_stream;
 	int errno;
 
+	mapi_id_t id_folder;
+	mapi_id_t id_message;
+	MapiItem *item = NULL;
+	guint32 options = 0;
+
 	/* see if it is there in cache */
 
 	mi = (CamelMapiMessageInfo *) camel_folder_summary_uid (folder->summary, uid);
@@ -1173,11 +1208,6 @@ mapi_folder_get_message( CamelFolder *folder, const char *uid, CamelException *e
 		return NULL;
 	}
 
-	mapi_id_t id_folder;
-	mapi_id_t id_message;
-	MapiItem *item = NULL;
-	guint32 options = 0;
-
 	options = MAPI_OPTIONS_FETCH_ALL | MAPI_OPTIONS_FETCH_BODY_STREAM | MAPI_OPTIONS_GETBESTBODY ;
 	exchange_mapi_util_mapi_ids_from_uid (uid, &id_folder, &id_message);
 
@@ -1354,7 +1384,7 @@ mapi_expunge (CamelFolder *folder, CamelException *ex)
 				deleted_head = NULL;
 				deleted_head = deleted_items = g_slist_prepend (deleted_items, mid);
 			}
-			deleted_items_uid = g_slist_prepend (deleted_items_uid, uid);
+			deleted_items_uid = g_slist_prepend (deleted_items_uid, (gpointer) uid);
 		}
 		camel_message_info_free (info);
 	}
@@ -1404,7 +1434,7 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	CamelMapiStore *mapi_store= CAMEL_MAPI_STORE(source->parent_store);
 	CamelFolderChangeInfo *changes = NULL;
 
-	char *folder_id = NULL;
+	const gchar *folder_id = NULL;
 	int i = 0;
 
 	GSList *src_msg_ids = NULL;
